package com.atlassian.confluence.extra;

import net.sf.hibernate.Session;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.HibernateException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;
import java.util.Random;
import java.util.List;
import java.util.ArrayList;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

import org.springframework.orm.hibernate.SessionFactoryUtils;
import org.apache.log4j.Category;
import bucket.container.ContainerManager;

public class RandomPageServlet extends HttpServlet
{
    public static final Category log = Category.getInstance(RandomPageServlet.class);
    private static Random seededRandomGenerator;
    private static List pageIds;

    public void init(ServletConfig servletConfig) throws ServletException
    {
        super.init(servletConfig);

        log.info("Initialising Random Page Servlet ...");
        long currentTime = System.currentTimeMillis();
        initialisePageIdsList();
        log.info("Random Page Servlet Initlialised (" + (System.currentTimeMillis() - currentTime) + "ms).");
    }

    protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        Long seed = getSeed(request);

        if (seed != null)
            seededRandomGenerator = new Random(seed.longValue());

        response.sendRedirect(request.getContextPath() + "/pages/viewpage.action?pageId=" + pageIds.get(rand(pageIds.size())));
    }

    private Long getSeed(HttpServletRequest request)
    {
        Long seed = null;

        if (request.getParameter("seed") != null)
        {
            try
            {
                seed = Long.valueOf((request.getParameter("seed")));
            }
            catch (NumberFormatException e)
            {
                // invalid seed
            }
        }
        return seed;
    }

    private synchronized void initialisePageIdsList()
    {
        if (pageIds != null)
            return;
        pageIds = new ArrayList();

        SessionFactory sessionFactory = (SessionFactory) ContainerManager.getComponent("sessionFactory");
        Session session = SessionFactoryUtils.getSession(sessionFactory, true);

        Connection connection = null;
        Statement statement = null;
        ResultSet rs = null;

        try
        {
            connection = session.connection();
            statement = connection.createStatement();
            rs = statement.executeQuery("select CONTENTID from CONTENT where CONTENT_STATUS = 'current' AND CONTENTTYPE = 'PAGE' AND PREVVER IS NULL");

            while (rs.next())
                pageIds.add(rs.getString(1));
        }
        catch (HibernateException e)
        {
            log.error(e);
        }
        catch (SQLException e)
        {
            log.error(e);
        }
    }

    public static int rand(int n)
    {
        if (n == 0)
            return 0;

        return ((int) (random() * Integer.MAX_VALUE)) % n;
    }

    public static double random()
    {
        if (seededRandomGenerator != null)
            return seededRandomGenerator.nextDouble();
        else
            return Math.random();
    }
}
